/**
 ** track_targetExecTaskCodels.c
 **
 ** Codels called by execution task track_targetExecTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <portLib.h>
#include <math.h>
#include <stdio.h>

#include "server/track_targetHeader.h"

POSTER_ID pomPos;
POSTER_ID targetPos;
int not_seen_since; /* period */
int max_not_seen; /* period */


/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 *              S_track_target_POSTER_NOT_FOUND
 */

/* track_target_InitExec  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
track_target_InitExec(TRACK_TARGET_INIT *init_params, int *report)
{
	if (posterFind(init_params->posPoster, &pomPos) < 0) {
		*report = S_track_target_POSTER_NOT_FOUND;
		return END;
	}

	SDI_F->targetPos.numRef = 0;
	SDI_F->targetPos.nbPts = 0;
	SDI_F->platinePosRef.pan = 0.0;
	SDI_F->platinePosRef.tilt = 0.0;

	return ETHER;
}

/*------------------------------------------------------------------------
 * Track
 *
 * Description: 
 *
 * Reports:      OK
 *              S_track_target_POSTER_NOT_FOUND
 *              S_track_target_TARGET_NOT_FOUD
 *              S_track_target_TARGET_LOST
 */

/* track_target_TrackStart  -  codel START of Track
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
track_target_TrackStart(TRACK_TARGET_TRACK *track_params, int *report)
{
	if (posterFind(track_params->posterName, &targetPos) < 0) {
		*report = S_track_target_POSTER_NOT_FOUND;
		return END;
	}

	GENPOS_TRAJ_POINTS pts;
	if (posterRead(targetPos, 0, &pts, sizeof(pts)) < 0) {
		*report = S_track_target_TARGET_NOT_FOUD;
		return END;
	}

	if (pts.nbPts == 0) {
		*report = S_track_target_TARGET_NOT_FOUD;
		return END;
	}

	not_seen_since = 0;
	max_not_seen = track_params->timeout / EXEC_TASK_PERIOD(TRACK_TARGET_EXECTASK_NUM);
	fprintf(stderr, "max_not_seen %d", max_not_seen);

	return EXEC;
}

static void track_interrupt()
{
	SDI_F->platinePosRef.pan = 0.0;
	SDI_F->platinePosRef.tilt = 0.0;
	SDI_F->targetPos.numRef++;
	SDI_F->targetPos.nbPts = 0;
}

/* track_target_TrackExec  -  codel EXEC of Track
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
track_target_TrackExec(TRACK_TARGET_TRACK *track_params, int *report)
{
	GENPOS_TRAJ_POINTS pts;
	if (posterRead(targetPos, 0, &pts, sizeof(pts)) < 0) {
		not_seen_since++;
	} else if (pts.nbPts == 0) {
		not_seen_since++;
	} else {
		/* big heuristic, just recopy the target position :D */
		SDI_F->targetPos.numRef++;
		SDI_F->targetPos.nbPts = 1;
		SDI_F->targetPos.points[0].x = pts.points[0].x;
		SDI_F->targetPos.points[0].y = pts.points[0].y;
		not_seen_since = 0;
	}

	if (not_seen_since >= max_not_seen) {
		track_interrupt();
		*report = S_track_target_TARGET_LOST;
		return END;
	}

	POM_POS pos;
	posterRead(pomPos, 0, &pos, sizeof(pos));
	double diff_x = SDI_F->targetPos.points[0].x - pos.mainToOrigin.euler.x;
	double diff_y = SDI_F->targetPos.points[0].y - pos.mainToOrigin.euler.y;
	double yaw = atan2(diff_y, diff_x);
	double delta_yaw = yaw - pos.mainToOrigin.euler.yaw;

	/* put it in the range [-M_PI, M_PI] */
	if (delta_yaw > M_PI)    
		delta_yaw -= 2*M_PI;
	if (delta_yaw < -M_PI)
		delta_yaw += 2*M_PI;

	SDI_F->platinePosRef.pan = delta_yaw;

	return EXEC;
}

/* track_target_TrackEnd  -  codel END of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
track_target_TrackEnd(TRACK_TARGET_TRACK *track_params, int *report)
{
	track_interrupt();
	return ETHER;
}

/* track_target_TrackInter  -  codel INTER of Track
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
track_target_TrackInter(TRACK_TARGET_TRACK *track_params, int *report)
{
	track_interrupt();
	return ETHER;
}


